#include "ast.hpp"

std::string UnitASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(unit (";
    for (Container::const_iterator it = nodesList_.begin(); it != nodesList_.end(); ++it)
        oss << **it << " ";
    oss << ")";
    return oss.str();
}

std::string IntNumberASTNode::toString() const
{
    std::ostringstream oss;
    oss << val_;
    return oss.str();
}

std::string DoubleNumberASTNode::toString() const
{
    std::ostringstream oss;
    oss << val_;
    return oss.str();
}

std::string ThisASTNode::toString() const
{
    return "this";
}

std::string VariableDeclASTNode::toString() const
{
    std::ostringstream oss;
    oss << "var " << name_ << " of " << type_ ;

    if(size_ != 0)
        oss << "[" << size_ << "]";

    return oss.str();
}

std::string VariableDeleteASTNode::toString() const
{
    std::ostringstream oss;
    oss << "delete " << name_;

    return oss.str();
}

std::string VariableASTNode::toString() const
{
    std::ostringstream oss;
    oss << name_;
    return oss.str();
}

std::string ArraySubscriptingASTNode::toString() const
{
    std::ostringstream oss;
    oss << *var_ << "[" << *index_ << "]";
    return oss.str();
}

std::string ReturnASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(return " << *rhs_ << ")";
    return oss.str();
}

std::string UnaryOpASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(" << op_ << " " << *child_ << ")";
    return oss.str();
}

std::string BinaryOpASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(" << op_ << " " << *lhs_ << " " << *rhs_ << ")";
    return oss.str();
}

std::string FuncCallASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(call " << functionName_ << " " << *params_ << ")";
    return oss.str();
}

std::string ConstructorDefASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(constructor " << *params_ << ")";
    return oss.str();
}

std::string FuncDefASTNode::toString() const
{
    std::ostringstream oss;
    if (extern_) 
        oss << "(extern " << functionName_ << " " << *params_ << ")" << " of" << return_type_;
    else
        oss << "(def " << functionName_ << " " << *params_ << " " << *body_ << ")" << " of" << return_type_;

    if(returns_array_)
        oss << "[]";
    return oss.str();
}

std::string IfASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(if " << *cond_ << " " << *if_body_;

    if(else_body_ != 0)
        oss << " else " << *else_body_;

    oss << ")";
    return oss.str();
}

std::string WhileASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(while " << *cond_ << " " << *body_ << ")";
    return oss.str();
}

std::string ForASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(for " << *begin_ << "; " << *cond_ << "; " << *step_ << " " << *body_ << ")";
    return oss.str();
}

std::string ClassDefASTNode::toString() const
{
    std::ostringstream oss;
    oss << "(class " << name_ << " ( " << *body_ << " ) ";
    return oss.str();
}

std::string ClassBodyASTNode::toString() const
{
    std::ostringstream oss;
    oss << "( " << fields_ << methods_ << " ) ";
    return oss.str();
}

std::string FieldASTNode::toString() const
{
    std::ostringstream oss;
    oss << "( " << *object_ << "->" << field_ << " ) ";
    return oss.str();
}

std::string MethodCallASTNode::toString() const
{
    std::ostringstream oss;
    oss << "( " << *object_ << "->" 
        << function_name_ << " " << *params_ << " ) ";
    return oss.str();
}
