#include <iostream>
#include "driver.hpp"

#include "llvm/DerivedTypes.h"
#include "llvm/LLVMContext.h"
#include "llvm/Module.h"
#include "llvm/Analysis/Verifier.h"
#include "llvm/Support/IRBuilder.h"
#include "llvm/Linker.h"
#include "llvm/Support/PathV1.h"

Module* theModule;
TypeFactory* typeFactory;
IRBuilder<>* builder;

int main(int argc, const char* argv[])
{
    Driver driver;
    LLVMContext& context = getGlobalContext();
    theModule = new Module("compiler_module", context);
    typeFactory = new TypeFactory();
    builder = new IRBuilder<>(getGlobalContext());

    Linker* linker = new Linker("compiler", theModule);
    bool is_native = true;
    linker->LinkInFile(sys::Path("./stdio.bc", 10), is_native);

    for (++argv; argv[0]; ++argv)
        if (*argv == std::string ("-p"))
            driver.trace_parsing = true;
        else if (*argv == std::string ("-s"))
            driver.trace_scanning = true;
        else if (!driver.parse(*argv)) {
            std::cout << "; AST: " << *driver.result << std::endl;
            theModule->dump();
        }
  
    return 0;
}
