%{
#include <iostream>
#include "parser.hpp"
#include "driver.hpp"

typedef yy::Parser::token token;

#define yyterminate() return token::END
%}

%option noyywrap
%option debug

digit [0-9]
char  [a-zA-Z_]
nchar [a-zA-Z_0-9]
ws    [ \t]
nl    [\r\n]

%%

"->"                return token::ARROW;    
"<"                 return token::LT;    
"<="                return token::LE;    
">"                 return token::GT;    
">="                return token::GE;    
"=="                return token::EQ;    
"!="                return token::NE;    
"!"                 return token::NOT;    
"+"                 return token::PLUS;     
"-"                 return token::MINUS;    
"*"                 return token::TIMES;    
"/"                 return token::SLASH;    
"("                 return token::LPAREN;   
")"                 return token::RPAREN;   
"="                 return token::ASSIGN;
","                 return token::COMMA;
";"                 return token::SEMICOLON;
":"                 return token::COLON;
"if"                return token::IF;
"else"              return token::ELSE;
"while"             return token::WHILE;
"for"               return token::FOR;
"def"               return token::DEF;
"of"                return token::OF;
"extern"            return token::EXTERN;
"class"             return token::CLASS;
"return"            return token::RETURN;
"delete"            return token::DELETE;
"virtual"           return token::VIRT;
"{"                 return token::BLOCK_BEGINS;
"}"                 return token::BLOCK_ENDS;
"["                 return token::LBRACKET;
"]"                 return token::RBRACKET;

{char}{nchar}*       {
                        yylval->sval = new std::string(yytext);
                        return token::IDENTIFIER;
                     }
-?{digit}+\.{digit}+ {
                        yylval->dval = std::atof(yytext);
                        return token::DOUBLENUMBER;
                     }
-?{digit}+           {
                        yylval->ival = std::atoi(yytext);
                        return token::INTNUMBER;
                     }
({nl}|{ws})+         /* ignore */
.                    { driver.error("invalid character"); }

%%

void Driver::scan_begin()
{
    yy_flex_debug = trace_scanning;
    if (filename == "-")
        yyin = stdin;
    else if (!(yyin = fopen (filename.c_str (), "r")))
    {
        error(std::string ("cannot open ") + filename);
        exit(1);
    }
}

void Driver::scan_end()
{
    fclose(yyin);
}
