#include "type_factory.h"

LangType* LangType::classOfField(const std::string& field) const
{
    std::queue<const LangType*> bfs_queue;
    bfs_queue.push(this);

    while(! bfs_queue.empty()) {
        const LangType* t = bfs_queue.front();
        bfs_queue.pop();

        if(t->fields.find(field) != t->fields.end())
            return const_cast<LangType*>(t);

        for(size_t i = 0; i < t->parents.size(); ++i)
            bfs_queue.push(t->parents[i]);
    }

    return NULL;
}

LangType* LangType::classOfMethod(const std::string& name) const
{
    std::queue<const LangType*> bfs_queue;
    bfs_queue.push(this);

    while(! bfs_queue.empty()) {
        const LangType* t = bfs_queue.front();
        bfs_queue.pop();

        if(t->methods.find(name) != t->methods.end())
            return const_cast<LangType*>(t);

        for(size_t i = 0; i < t->parents.size(); ++i)
            bfs_queue.push(t->parents[i]);
    }

    return NULL;
}

Value* LangType::castTo(Value* object, LangType* target_type) const
{
    if(this == target_type)
        return object;

    if(object->getType() == target_type->llvm_type)
        return object;

    std::map<LangType*, size_t>::const_iterator it = 
        castOffsets.find(target_type);

    if(it == castOffsets.end()) 
        return errorV("Unknown cast.");;
    size_t offset = it->second;

    if(offset == 0) {
        return builder->CreateBitCast(object, target_type->llvm_type);
    }

    Value* i8ptr = builder->CreateBitCast(
            object,
            Type::getInt8PtrTy(getGlobalContext())
    );

    return builder->CreateBitCast(
        builder->CreateGEP(
            i8ptr, ConstantInt::get(
                getGlobalContext(), APInt(64, offset)
            )
        ), target_type->llvm_type
    );
}


Value* LangType::downcast(Value* object, LangType* object_type) const
{
    if(object_type == this)
        return object;

    std::map<LangType*, size_t>::const_iterator it = 
        castOffsets.find(object_type);

    if(it == castOffsets.end()) 
        return errorV("Unknown cast.");;
    size_t offset = it->second;

    if(offset == 0) {
        return builder->CreateBitCast(object, llvm_type);
    }

    Value* i8ptr = builder->CreateBitCast(
            object,
            Type::getInt8PtrTy(getGlobalContext())
    );

    return builder->CreateBitCast(
        builder->CreateGEP(
            i8ptr, ConstantInt::get(
                getGlobalContext(), APInt(64, -offset)
            )
        ), llvm_type
    );
    
}

LangType* TypeFactory::getType(const std::string& type) const
{
    types_map_t::const_iterator it = types_map_.find(type);
    return it != types_map_.end() ? it->second : NULL; 
}

void TypeFactory::addType(const LangType& new_type)
{
    types_map_[new_type.type_name] = new LangType(new_type);
}
