#ifndef TYPE_FACTORY_H
#define TYPE_FACTORY_H

#include "llvm/DerivedTypes.h"
#include "llvm/LLVMContext.h"
#include "llvm/Module.h"
#include "llvm/Analysis/Verifier.h"
#include "llvm/Support/IRBuilder.h"

#include "llvm/Analysis/ProfileInfo.h"
#include "llvm/Support/raw_ostream.h"

#include <map>
#include <vector>
#include <queue>

using namespace llvm;

extern Value* errorV(const char *str);
extern IRBuilder<>* builder;

struct LangType;

struct VTEntry
{
    LangType* base_class;
    LangType* class_owner;
    Type* return_type;
    std::vector<Type*>* args;
    size_t offset;
};

typedef std::map<std::string, VTEntry> vtable_t;

struct LangType
{
    std::string type_name;
    Type* llvm_type;
    PointerType* llvm_ptr_type;
    size_t size;
    std::map<std::string, std::string> methods;
    std::map<std::string, Value*> fields;
    std::map<LangType*, vtable_t> parent_vtables;
    vtable_t vtable;
    std::map<LangType*, size_t> castOffsets;
    std::vector<LangType*> parents;

    LangType* classOfField(const std::string& field) const;
    LangType* classOfMethod(const std::string& name) const;
    Value* castTo(Value* object, LangType* target_type) const;
    Value* downcast(Value* object, LangType* object_type) const;
};

struct Variable
{
    AllocaInst* alloca_;
    LangType* type_name;
};

class TypeFactory
{
    typedef std::map<std::string, LangType*> types_map_t;

    types_map_t types_map_;

    public:
    TypeFactory()
    {
        const LangType built_in_types[] = {
            { 
                "int", 
                Type::getInt64Ty(getGlobalContext()),
                Type::getInt64PtrTy(getGlobalContext()),
                sizeof(long)
            },
            { 
                "double", 
                Type::getDoubleTy(getGlobalContext()),
                Type::getDoublePtrTy(getGlobalContext()),
                sizeof(double)
            }
        };

        const int built_in_types_cnt = 
            sizeof(built_in_types) / sizeof(struct LangType);

        for(size_t i = 0; i < built_in_types_cnt; ++i)
            addType(built_in_types[i]);
    }

    LangType* getType(const std::string& type) const;
    void addType(const struct LangType& new_type);
};

#endif // TYPE_FACTORY_H
